@@ -19,6 +19,7 @@
 import copy
 from functools import partial
 import os
+import re
 
 # GPU initialization (Windows only).
 # Needs to be up here to load before tensorflow is imported through the pyjamas imports.
@@ -35,6 +36,7 @@ os.environ["CRYPTOGRAPHY_OPENSSL_NO_LEGACY"] = "1"  # gets around compatibility
 import sys
 from typing import List, Tuple, Optional
 from enum import IntEnum
+import tomlkit
 
 import numpy
 from PyQt6 import QtCore, QtGui, QtWidgets
@@ -85,6 +87,12 @@ class PyJAMAS(QtCore.QObject):
     folder = os.path.split(__file__)[0]
     logo_path = os.path.join(folder, logo_filename)
 
+    # Config path
+    config_filename = 'config.toml'
+    config_path = os.path.join(folder, config_filename)
+    default_config_filename = 'default_config.toml'
+    default_config_path = os.path.join(folder, default_config_filename)
+
     # Annotation modes.
     no_annotations: int = 0
     fiducial_annotations: int = 1
@@ -112,6 +120,7 @@ class PyJAMAS(QtCore.QObject):
     matlab_extension: str = '.mat'
     image_extensions: Tuple[str] = rimcore.rimage.image_extensions
     plugin_extension: str = '.py'
+    config_extension: str = '.toml'
     notebook_extension: str = '.ipynb'
     classifier_extension: str = '.cfr'
     backgroundimage_extension: str = '.bg'
@@ -127,17 +136,26 @@ class PyJAMAS(QtCore.QObject):
     polyline_brush_style: QtGui.QBrush = QtGui.QBrush()
     fiducial_font: QtGui.QFont = QtGui.QFont('Arial', 8)
 
-    zoom_factors: tuple = (1., 2., 4., 8., .125, .25, .5)
-
+    # default options
+    brush_size: int = 3
+    display_fiducial_ids: bool = False
+    fps: int = 7
+    cwd: str = os.getcwd()
+    margin_size: int = 0
+    balloon_crop_size: int = 50
+    crop_tracked_polyline: bool = False
+    darktheme: bool = False
+    close_all_polylines: bool = False
     livewire_margin: int = 5
-    livewire_gaussian_sigma: float = 0.
-    balloon_crop_size: int = 50  # crop size used to inflate balloons
-    livewire_shortest_path_fn = tuple(rimcore.rimage.livewire_shortest_path_fns.values())[0]  # fn to calculate shortest path between two pixels
+    livewire_gaussian_sigma: float = 5.0
+    livewire_shortest_path_fn: str = 'Dijkstra'
+
+    zoom_factors: tuple = (1., 2., 4., 8., .125, .25, .5)
 
     undo_stack_size: int = 50
 
     # Read version.
-    __version__: str = '2025.5.0'
+    __version__: str = '2025.10.1'
 
     def __init__(self):
         self.initData()  # Initialize object variables.
@@ -179,7 +197,7 @@ class PyJAMAS(QtCore.QObject):
         self.timeSlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
         self.timeSlider.setObjectName("timeSlider")
 
-        self.darktheme = False
+        self.darktheme: bool = self.config["options"].get("darktheme", PyJAMAS.darktheme)
 
         self.menubar = QtWidgets.QMenuBar(self.MainWindow)
         self.menubar.setGeometry(QtCore.QRect(0, 0, 1183, 22))
@@ -224,400 +242,374 @@ class PyJAMAS(QtCore.QObject):
         # Time slider and default image
         self.timeSlider.valueChanged.connect(self.image.cbTimeSlider)
         self.io.cbLoadTimeSeries(PyJAMAS.logo_path)
+        self.cwd = self.config["options"].get("cwd", PyJAMAS.cwd)  # so cwd is not overwritten when default image is loaded
+        if self.cwd == "":  # Default config loaded
+            self.cwd = os.getcwd()
 
         # IO MENU ----------------------------------------------------------------------
         # Load image action
-        self.addMenuItem(self.menuIO, 'Load image ...', 't',
-                         self.io.cbLoadTimeSeries)
+        self.addMenuItem(self.menuIO, 'Load image ...', self.io.cbLoadTimeSeries)
 
         # Save time-lapse.
-        self.addMenuItem(self.menuIO, 'Save image ...', 's',
-                         self.io.cbSaveTimeSeries)
+        self.addMenuItem(self.menuIO, 'Save image ...', self.io.cbSaveTimeSeries)
 
         # Save time-lapse, but only ROI inside the polygon.
-        self.addMenuItem(self.menuIO, 'Save image in polyline to working folder', 'u',
+        self.addMenuItem(self.menuIO, 'Save image in polyline to working folder',
                          partial(self.io.cbSaveROI, filename=''))
 
         self.menuIO.addSeparator()
 
         # Load annotations.
-        self.addMenuItem(self.menuIO,'Load annotations ...', 'Shift+L',
-                         self.io.cbLoadAnnotations)
+        self.addMenuItem(self.menuIO, 'Load annotations ...', self.io.cbLoadAnnotations)
 
         self.addMenuItem(self.menuIO, 'Load annotations (additive) ...',
-                         'Alt+Shift+L',
                          lambda: self.io.cbLoadAnnotations(replace=False))
 
         # Save annotations.
-        self.addMenuItem(self.menuIO, 'Save annotations ...', 'a',
-                         self.io.cbSaveAnnotations)
+        self.addMenuItem(self.menuIO, 'Save annotations ...', self.io.cbSaveAnnotations)
 
-        self.addMenuItem(self.menuIO, 'Export individual fiducial-polyline annotations', None,
+        self.addMenuItem(self.menuIO, 'Export individual fiducial-polyline annotations',
                          self.io.cbExportPolylineAnnotations)
 
-        self.addMenuItem(self.menuIO, 'Export ALL fiducial-polyline annotations ...', None,
+        self.addMenuItem(self.menuIO, 'Export ALL fiducial-polyline annotations ...',
                          self.io.cbExportAllPolylineAnnotations)
 
         self.menuIO.addSeparator()
 
-        self.addMenuItem(self.menuIO, 'Load classifier ...', None,
-                         self.io.cbLoadClassifier)
+        self.addMenuItem(self.menuIO, 'Load classifier ...', self.io.cbLoadClassifier)
 
-        self.addMenuItem(self.menuIO, 'Save current classifier ...', None,
-                         self.io.cbSaveClassifier)
+        self.addMenuItem(self.menuIO, 'Save current classifier ...', self.io.cbSaveClassifier)
 
         self.menuIO.addSeparator()
 
-        self.addMenuItem(self.menuIO, 'Import mask as polylines ...', None, self.io.cbImportMasks)
+        self.addMenuItem(self.menuIO, 'Import mask as polylines ...', self.io.cbImportMasks)
 
         # Export roi and  masks.
-        self.addMenuItem(self.menuIO, 'Export ROI and binary masks ...', 'Shift+M',
+        self.addMenuItem(self.menuIO, 'Export ROI and binary masks ...',
                          partial(self.io.cbExportROIAndMasks, polyline=None))
 
         # Export binary image roi.
-        self.addMenuItem(self.menuIO, 'Export current ROIs as binary image ...', 'b',
+        self.addMenuItem(self.menuIO, 'Export current ROIs as binary image ...',
                          self.io.cbExportCurrentAnnotationsBinaryImage)
 
         self.menuIO.addSeparator()
 
         # Import SIESTA annotations.
-        self.addMenuItem(self.menuIO, 'Import SIESTA annotations ...', None,
-                         self.io.cbImportSIESTAAnnotations)
+        self.addMenuItem(self.menuIO, 'Import SIESTA annotations ...', self.io.cbImportSIESTAAnnotations)
 
         # Add SIESTA annotations.
-        self.addMenuItem(self.menuIO, 'Import SIESTA annotations (additive) ...', None,
+        self.addMenuItem(self.menuIO, 'Import SIESTA annotations (additive) ...',
                          lambda: self.io.cbImportSIESTAAnnotations(replace=False))
 
         # Export SIESTA annotations.
-        self.addMenuItem(self.menuIO, 'Export SIESTA annotations ...', None,
-                         self.io.cbExportSIESTAAnnotations)
+        self.addMenuItem(self.menuIO, 'Export SIESTA annotations ...', self.io.cbExportSIESTAAnnotations)
 
         self.menuIO.addSeparator()
 
         # Save display.
-        self.addMenuItem(self.menuIO, 'Save display ...', None, self.io.cbSaveDisplay)
+        self.addMenuItem(self.menuIO, 'Save display ...', self.io.cbSaveDisplay)
 
         # Save annotations movie.
-        self.addMenuItem(self.menuIO, 'Export movie with annotations ...', None, self.io.cbExportMovie)
+        self.addMenuItem(self.menuIO, 'Export movie with annotations ...', self.io.cbExportMovie)
 
         self.menubar.addMenu(self.menuIO)
 
+        self.menuIO.addSeparator()
+
+        # Load configuration from file.
+        self.addMenuItem(self.menuIO, 'Load config ...', self.io.cbLoadConfig)
+
+        # Save configuration to file.
+        self.addMenuItem(self.menuIO, 'Save config ...', self.io.cbSaveConfig)
+
         # OPTIONS MENU ----------------------------------------------------------------------
         # Set brush size.
 
-        self.addMenuItem(self.menuOptions, 'Undo',
-                         'Ctrl+Z',
-                         self.options.cbUndo)
+        self.addMenuItem(self.menuOptions, 'Undo', self.options.cbUndo)
 
         self.menuOptions.addSeparator()
 
-        self.addMenuItem(self.menuOptions, 'Set brush size ...', None,
-                         self.options.cbSetBrushSize)
+        self.addMenuItem(self.menuOptions, 'Brush size ...', self.options.cbSetBrushSize)
 
-        self.addMenuItem(self.menuOptions, 'Close all loaded polylines', None,
-                         self.options.cbCloseAllPolylines)
+        self.addMenuItem(self.menuOptions, 'Crop margins ...', self.options.cbSetMarginSize)
 
-        self.addMenuItem(self.menuOptions, 'Display fiducial and polyline ids', 'i',
-                         self.options.cbDisplayFiducialIDs, )
-
-        self.addMenuItem(self.menuOptions, 'Set frames per sec ...', None,
+        self.addMenuItem(self.menuOptions, 'Frames per sec ...',
                          self.options.cbFramesPerSec)
 
-        self.addMenuItem(self.menuOptions, 'Set working folder ...', None, self.options.cbSetCWD)
+        self.addMenuItem(self.menuOptions, 'Working folder ...', self.options.cbSetCWD)
 
-        self.addMenuItem(self.menuOptions, 'Set crop margins ...', None, self.options.cbSetMarginSize)
+        self.menuOptions.addSeparator()
 
-        self.addMenuItem(self.menuOptions, 'Set balloon crop size ...', None,
-                         self.options.cbSetImageProcessingCropSize)
+        self.addMenuItem(self.menuOptions, 'Balloon crop size ...', self.options.cbSetImageProcessingCropSize)
 
-        self.addMenuItem(self.menuOptions, 'LiveWire Gaussian smoothing ...', None,
-                         self.options.cbSetLiveWireSmoothGaussian)
+        self.addMenuItem(self.menuOptions, 'LiveWire Gaussian smoothing ...', self.options.cbSetLiveWireSmoothGaussian)
 
-        self.addMenuItem(self.menuOptions, 'LiveWire shortest path function ...', None,
-                         self.options.cbSetLivewireShortestPathFunction)
+        self.addMenuItem(self.menuOptions, 'LiveWire shortest path function ...', self.options.cbSetLivewireShortestPathFunction)
 
-        self.addMenuItem(self.menuOptions, 'Toggle crop tracked polyline', None, self.options.cbCropTracked)
+        self.menuOptions.addSeparator()
 
-        self.addMenuItem(self.menuOptions, 'Dark/light display theme', None,
-                         self.options.cbChangeDisplayTheme)
+        self.addMenuItem(self.menuOptions, 'Close all loaded polylines', self.options.cbCloseAllPolylines)
 
-        self.addMenuItem(self.menuOptions, 'Toggle highlighting objects under cursor', None,
+        self.addMenuItem(self.menuOptions, 'Display fiducial and polyline ids', self.options.cbDisplayFiducialIDs)
+
+        self.addMenuItem(self.menuOptions, 'Toggle crop tracked polyline', self.options.cbCropTracked)
+
+        self.addMenuItem(self.menuOptions, 'Toggle highlighting objects under cursor',
                          self.options.cbToggleHighlightOnHover)
 
+        self.addMenuItem(self.menuOptions, 'Change dilation/erosion footprint', self.options.cbChangeFootprint)
+
+        self.menuOptions.addSeparator()
+
+        self.addMenuItem(self.menuOptions, 'Dark/light display theme', self.options.cbChangeDisplayTheme)
+
+        self.menuOptions.addSeparator()
+
+        self.addMenuItem(self.menuOptions, 'Change key bindings ...', self.options.cbChangeKeyBindings)
+
         self.menubar.addMenu(self.menuOptions)
 
         # IMAGE MENU ----------------------------------------------------------------------
 
         # Adjust contrast
-        self.addMenuItem(self.menuImage, 'Adjust contrast ...', 'Shift+C',
-                         self.image.cbAdjustContrast)
+        self.addMenuItem(self.menuImage, 'Adjust contrast ...', self.image.cbAdjustContrast)
 
         self.menuImage.addSeparator()
 
         # Rotate image CW.
-        self.addMenuItem(self.menuImage, 'Play sequence', '\\', self.image.cbPlay)
+        self.addMenuItem(self.menuImage, 'Play sequence', self.image.cbPlay)
 
         self.menuImage.addSeparator()
 
-        self.addMenuItem(self.menuImage, 'Rotate 90 degree clockwise', 'Shift+Right',
-                         partial(self.image.cbRotateImage,
-                                 direction=rcallbacks.rcbimage.RCBImage.CW))
+        self.addMenuItem(self.menuImage, 'Rotate 90 degree clockwise',
+                         partial(self.image.cbRotateImage, direction=rcallbacks.rcbimage.RCBImage.CW))
 
         # Rotate image CCW.
-        self.addMenuItem(self.menuImage, 'Rotate 90 degree counter-clockwise', 'Shift+Left',
-                         partial(self.image.cbRotateImage,
-                                 direction=rcallbacks.rcbimage.RCBImage.CCW))
+        self.addMenuItem(self.menuImage, 'Rotate 90 degree counter-clockwise',
+                         partial(self.image.cbRotateImage, direction=rcallbacks.rcbimage.RCBImage.CCW))
 
         # Flip left-right.
-        self.addMenuItem(self.menuImage, 'Flip left-right', 'Alt+Right',
-                         partial(self.image.cbFlipImage,
-                                 direction=rcallbacks.rcbimage.RCBImage.LEFT_RIGHT))
+        self.addMenuItem(self.menuImage, 'Flip left-right',
+                         partial(self.image.cbFlipImage, direction=rcallbacks.rcbimage.RCBImage.LEFT_RIGHT))
 
         # Flip top-bottom.
-        self.addMenuItem(self.menuImage, 'Flip up-down', 'Alt+Up',
-                         partial(self.image.cbFlipImage,
-                                 direction=rcallbacks.rcbimage.RCBImage.UP_DOWN))
+        self.addMenuItem(self.menuImage, 'Flip up-down',
+                         partial(self.image.cbFlipImage, direction=rcallbacks.rcbimage.RCBImage.UP_DOWN))
 
         self.menuImage.addSeparator()
 
         # Invert image.
-        self.addMenuItem(self.menuImage, 'Invert image', None, self.image.cbInvertImage)
+        self.addMenuItem(self.menuImage, 'Invert image', self.image.cbInvertImage)
 
         # Smoothen image.
-        self.addMenuItem(self.menuImage, 'Gaussian smoothing ...', None, self.image.cbGaussianImage)
+        self.addMenuItem(self.menuImage, 'Gaussian smoothing ...', self.image.cbGaussianImage)
 
         # Gradient.
-        self.addMenuItem(self.menuImage, 'Gradient', None, self.image.cbGradientImage)
+        self.addMenuItem(self.menuImage, 'Gradient', self.image.cbGradientImage)
 
         # Maximum intensity projection.
-        self.addMenuItem(self.menuImage, 'Maximum intensity projection', None,
-                         partial(self.image.cbProjectImage,
-                                 projection_type=rcallbacks.rcbimage.projection_types.MAX))
+        self.addMenuItem(self.menuImage, 'Maximum intensity projection',
+                         partial(self.image.cbProjectImage, projection_type=rcallbacks.rcbimage.projection_types.MAX))
 
         # Sum intensity projection.
-        self.addMenuItem(self.menuImage, 'Sum intensity projection', None,
-                         partial(self.image.cbProjectImage,
-                                 projection_type=rcallbacks.rcbimage.projection_types.SUM))
+        self.addMenuItem(self.menuImage, 'Sum intensity projection',
+                         partial(self.image.cbProjectImage, projection_type=rcallbacks.rcbimage.projection_types.SUM))
 
         self.menuImage.addSeparator()
 
-        self.addMenuItem(self.menuImage, 'Register image', 'Shift+R',
-                         self.image.cbRegisterImage)
+        self.addMenuItem(self.menuImage, 'Register image', self.image.cbRegisterImage)
 
-        self.addMenuItem(self.menuImage, 'Rescale image ...', None,
-                         self.image.cbRescaleImage)
+        self.addMenuItem(self.menuImage, 'Rescale image ...', self.image.cbRescaleImage)
 
         self.menuImage.addSeparator()
         # Zoom action.
-        self.addMenuItem(self.menuImage, 'Zoom', 'z', self.image.cbZoom)
+        self.addMenuItem(self.menuImage, 'Zoom', self.image.cbZoom)
 
         # Crop action.
-        self.addMenuItem(self.menuImage, 'Crop', 'Shift+P',
-                         partial(self.image.cbCrop, polyline=None))
+        self.addMenuItem(self.menuImage, 'Crop', partial(self.image.cbCrop, polyline=None))
 
         # Kymograph.
-        self.addMenuItem(self.menuImage, 'Kymograph', None, partial(self.image.cbKymograph, polyline=None))
+        self.addMenuItem(self.menuImage, 'Kymograph', partial(self.image.cbKymograph, polyline=None))
 
         # Orthogonal views.
-        self.addMenuItem(self.menuImage, 'Orthogonal views', 'o', self.image.cbOrthogonalViews)
+        self.addMenuItem(self.menuImage, 'Orthogonal views', self.image.cbOrthogonalViews)
 
         self.menuImage.addSeparator()
 
         # Find seeds.
-        self.addMenuItem(self.menuImage, 'Find seeds ...', 'Shift+S',
-                         self.image.cbFindSeeds)
+        self.addMenuItem(self.menuImage, 'Find seeds ...', self.image.cbFindSeeds)
 
         # Add fiducials in the centre of polylines.
-        self.addMenuItem(self.menuImage, 'Add seeds in polyline centroids ...', None,
-                         self.image.cbCentroidSeeds)
+        self.addMenuItem(self.menuImage, 'Add seeds in polyline centroids ...', self.image.cbCentroidSeeds)
 
         # Propagate seeds.
-        self.addMenuItem(self.menuImage, 'Propagate seeds ...', 'Shift+E',
-                         self.image.cbPropagateSeeds)
+        self.addMenuItem(self.menuImage, 'Propagate seeds ...', self.image.cbPropagateSeeds)
 
         # Expand seeds.
-        self.addMenuItem(self.menuImage, 'Expand seeds ...', 'Shift+X',
-                         self.image.cbExpandSeeds)
+        self.addMenuItem(self.menuImage, 'Expand seeds ...', self.image.cbExpandSeeds)
 
         # Expand and propagate seeds.
-        self.addMenuItem(self.menuImage, 'Expand and propagate seeds ...', 'Shift+Z',
-                         self.image.cbExpandNPropagateSeeds)
+        self.addMenuItem(self.menuImage, 'Expand and propagate seeds ...', self.image.cbExpandNPropagateSeeds)
 
         # Propagate and expand balloons.
-        self.addMenuItem(self.menuImage, 'Propagate balloons ...', None,
-                         self.image.cbPropagateBalloons)
+        self.addMenuItem(self.menuImage, 'Propagate balloons ...', self.image.cbPropagateBalloons)
 
         # Find puncta.
-        self.addMenuItem(self.menuImage, 'Find puncta ...', 'p',
-                         self.image.cbFindPuncta)
+        self.addMenuItem(self.menuImage, 'Find puncta ...', self.image.cbFindPuncta)
 
         # Classifiers.
         self.menuImage.addMenu(self.menuClassifiers)
         # -- Display Classifier: YellowBrick?
-        self.addMenuItem(self.menuClassifiers, 'Create and train logistic regression model ...', None,
+        self.addMenuItem(self.menuClassifiers, 'Create and train logistic regression model ...',
                          self.classifiers.cbCreateLR)
-        self.addMenuItem(self.menuClassifiers, 'Create and train support vector machine ...', None,
+        self.addMenuItem(self.menuClassifiers, 'Create and train support vector machine ...',
                          self.classifiers.cbCreateSVM)
         # self.addMenuItem(self.menuClassifiers, 'Create and train convolutional neural network ...', None,
         #                 self.classifiers.cbCreateCNN)
-        self.addMenuItem(self.menuClassifiers, 'Create and train UNet ...', None,
-                         self.classifiers.cbCreateUNet)
-        self.addMenuItem(self.menuClassifiers, 'Create and train ReSCUNet ...', None,
-                         self.classifiers.cbCreateReSCUNet)
+        self.addMenuItem(self.menuClassifiers, 'Create and train UNet ...', self.classifiers.cbCreateUNet)
+        self.addMenuItem(self.menuClassifiers, 'Create and train ReSCUNet ...', self.classifiers.cbCreateReSCUNet)
 
         # -- Decision Trees
 
         self.menuClassifiers.addSeparator()
 
-        self.addMenuItem(self.menuClassifiers, 'Apply classifier ...', None, self.classifiers.cbApplyClassifier)
+        self.addMenuItem(self.menuClassifiers, 'Apply classifier ...', self.classifiers.cbApplyClassifier)
 
-        self.addMenuItem(self.menuClassifiers, 'Apply classifier to current slice ...', 'Shift+A',
+        self.addMenuItem(self.menuClassifiers, 'Apply classifier to current slice ...',
                          lambda: self.classifiers.cbApplyClassifier(firstSlice=self.curslice + 1, lastSlice=self.curslice + 1))
 
         self.menuClassifiers.addSeparator()
 
-        self.addMenuItem(self.menuClassifiers, 'Non-maximum suppression ...', None,
-                         self.classifiers.cbNonMaxSuppression)
+        self.addMenuItem(self.menuClassifiers, 'Non-maximum suppression ...', self.classifiers.cbNonMaxSuppression)
 
-        self.addMenuItem(self.menuClassifiers, 'Segment detected objects ...', None,
-                         self.image.cbSegmentDetectedObjects)
+        self.addMenuItem(self.menuClassifiers, 'Segment detected objects ...', self.image.cbSegmentDetectedObjects)
 
-        self.addMenuItem(self.menuClassifiers, 'Change neural net step size ...', None,
+        self.addMenuItem(self.menuClassifiers, 'Change neural net step size ...',
                          self.classifiers.cbSetNeuralNetStepSize)
 
         self.menuImage.addSeparator()
 
         # Next image action
-        self.addMenuItem(self.menuImage, 'Next frame', '.', self.image.cbNextFrame)
+        self.addMenuItem(self.menuImage, 'Next frame', self.image.cbNextFrame)
 
         # Previous image action
-        self.addMenuItem(self.menuImage, 'Previous frame', ',', self.image.cbPrevFrame)
+        self.addMenuItem(self.menuImage, 'Previous frame', self.image.cbPrevFrame)
 
         # Beginning and end actions
-        self.addMenuItem(self.menuImage, 'Go to beginning', '1', partial(self.image.cbGoTo, slice_index=0))
-        self.addMenuItem(self.menuImage, 'Go to end', '0', partial(self.image.cbGoTo, slice_index=-1))
+        self.addMenuItem(self.menuImage, 'Go to beginning', partial(self.image.cbGoTo, slice_index=0))
+        self.addMenuItem(self.menuImage, 'Go to end', partial(self.image.cbGoTo, slice_index=-1))
 
         self.menuImage.addSeparator()
 
         # Display info.
-        self.addMenuItem(self.menuImage, 'Display info', 'Ctrl+I',
-                         self.image.cbDisplayInfo)
+        self.addMenuItem(self.menuImage, 'Display info', self.image.cbDisplayInfo)
 
         self.menubar.addMenu(self.menuImage)
 
         # ANNOTATIONS MENU ----------------------------------------------------------------------
         # No annotations.
-        self.addMenuItem(self.menuAnnotations, 'No annotations', 'n', self.annotations.cbNoAnn)
+        self.addMenuItem(self.menuAnnotations, 'No annotations', self.annotations.cbNoAnn)
 
-        self.addMenuItem(self.menuAnnotations, 'Hide/display annotations', 'h', self.annotations.cbHideAnn)
+        self.addMenuItem(self.menuAnnotations, 'Hide/display annotations', self.annotations.cbHideAnn)
 
         # Fiducials
-        self.addMenuItem(self.menuAnnotations, 'Fiducials', 'f', self.annotations.cbFiducials)
+        self.addMenuItem(self.menuAnnotations, 'Fiducials', self.annotations.cbFiducials)
 
         # Rectangles
-        self.addMenuItem(self.menuAnnotations, 'Rectangles', 'r', self.annotations.cbRectangles)
+        self.addMenuItem(self.menuAnnotations, 'Rectangles', self.annotations.cbRectangles)
 
         # Polylines
-        self.addMenuItem(self.menuAnnotations, 'Polylines', 'y', self.annotations.cbPolylines)
+        self.addMenuItem(self.menuAnnotations, 'Polylines', self.annotations.cbPolylines)
 
         # Livewire
-        self.addMenuItem(self.menuAnnotations, 'LiveWire', 'w', self.annotations.cbLiveWire)
+        self.addMenuItem(self.menuAnnotations, 'LiveWire', self.annotations.cbLiveWire)
 
         # Balloon
-        self.addMenuItem(self.menuAnnotations, 'Balloon', 'Shift+B', self.annotations.cbInflateBalloon)
+        self.addMenuItem(self.menuAnnotations, 'Balloon', self.annotations.cbInflateBalloon)
 
         self.menuAnnotations.addSeparator()
 
         # Copy and paste polyline
-        self.addMenuItem(self.menuAnnotations, 'Copy polyline', 'c', self.annotations.cbCopyPolyline)
+        self.addMenuItem(self.menuAnnotations, 'Copy polyline', self.annotations.cbCopyPolyline)
 
-        self.addMenuItem(self.menuAnnotations, 'Paste polyline', 'v', self.annotations.cbPastePolyline)
+        self.addMenuItem(self.menuAnnotations, 'Paste polyline', self.annotations.cbPastePolyline)
 
         # Move polyline.
-        self.addMenuItem(self.menuAnnotations, 'Move polyline', 'm', self.annotations.cbMovePolyline)
+        self.addMenuItem(self.menuAnnotations, 'Move polyline', self.annotations.cbMovePolyline)
 
         # Dilate/erode polyline
-        self.addMenuItem(self.menuAnnotations, 'Dilate/Erode polyline', None, self.annotations.cbDilateErodePolyline)
+        self.addMenuItem(self.menuAnnotations, 'Dilate/Erode polyline', self.annotations.cbDilateErodePolyline)
 
         self.menuAnnotations.addSeparator()
 
-        self.addMenuItem(self.menuAnnotations, 'Track fiducials ...', 'Shift+T',
-                         self.annotations.cbTrackFiducials)
+        self.addMenuItem(self.menuAnnotations, 'Track fiducials ...', self.annotations.cbTrackFiducials)
 
         self.menuAnnotations.addSeparator()
 
         # Delete polylines on the current image.
-        self.addMenuItem(self.menuAnnotations, 'Delete polylines on current frame', 'Backspace',
-                         self.annotations.cbDeleteSlicePoly)
+        self.addMenuItem(self.menuAnnotations, 'Delete polylines on current frame', self.annotations.cbDeleteSlicePoly)
 
         # Delete fiducials on current image.
-        self.addMenuItem(self.menuAnnotations, 'Delete fiducials on current frame', None,
+        self.addMenuItem(self.menuAnnotations, 'Delete fiducials on current frame',
                          self.annotations.cbDeleteSliceFiducials)
 
         # Delete annotations on the current image.
-        self.addMenuItem(self.menuAnnotations, 'Delete all annotations on current frame', '-',
+        self.addMenuItem(self.menuAnnotations, 'Delete all annotations on current frame',
                          self.annotations.cbDeleteSliceAnn)
 
         # Delete ALL annotations.
-        self.addMenuItem(self.menuAnnotations, 'Delete all annotations in the sequence', '_',
+        self.addMenuItem(self.menuAnnotations, 'Delete all annotations in the sequence',
                          self.annotations.cbDeleteAllAnn)
 
         # Delete ALL fiducials.
-        self.addMenuItem(self.menuAnnotations, 'Delete all fiducials in the sequence', None,
+        self.addMenuItem(self.menuAnnotations, 'Delete all fiducials in the sequence',
                          self.annotations.cbDeleteAllFiducials)
 
         # Delete fiducials OUTSIDE polyline.
         self.addMenuItem(self.menuAnnotations, 'Delete fiducials outside polyline',
-                         'Shift+O',
                          self.annotations.cbDeleteFiducialsOutsidePoly)
 
         self.menubar.addMenu(self.menuAnnotations)
 
         # Delete fiducials INSIDE polyline.
         self.addMenuItem(self.menuAnnotations, 'Delete fiducials inside polyline',
-                         'Shift+I',
                          self.annotations.cbDeleteFiducialsInsidePoly)
 
         self.menubar.addMenu(self.menuAnnotations)
 
         # MEASUREMENTS MENU --------------------------------------------------------------------
-        self.addMenuItem(self.menuMeasurements, 'Measure polylines ...', None, self.measurements.cbMeasurePoly)
+        self.addMenuItem(self.menuMeasurements, 'Measure polylines ...', self.measurements.cbMeasurePoly)
 
         self.menubar.addMenu(self.menuMeasurements)
 
         # BATCH MENU ---------------------------------------------------------------------------
         # Max project and concatenate.
-        self.addMenuItem(self.menuBatch, "Max project and concatenate ...", None,
-                         partial(self.batch.cbBatchProjectConcat,
-                                 projection_type=rcallbacks.rcbimage.projection_types.MAX))
+        self.addMenuItem(self.menuBatch, "Max project and concatenate ...",
+                         partial(self.batch.cbBatchProjectConcat, projection_type=rcallbacks.rcbimage.projection_types.MAX))
 
         # Sum project and concatenate.
-        self.addMenuItem(self.menuBatch, "Sum project and concatenate ...", None,
-                         partial(self.batch.cbBatchProjectConcat,
-                                 projection_type=rcallbacks.rcbimage.projection_types.SUM))
+        self.addMenuItem(self.menuBatch, "Sum project and concatenate ...",
+                         partial(self.batch.cbBatchProjectConcat, projection_type=rcallbacks.rcbimage.projection_types.SUM))
 
         # Resize images.
-        self.addMenuItem(self.menuBatch, "Resize images ...", None, self.batch.cbBatchResize)
+        self.addMenuItem(self.menuBatch, "Resize images ...", self.batch.cbBatchResize)
 
         # Resize images.
-        self.addMenuItem(self.menuBatch, "Correct images ...", None, self.batch.cbBatchFlatFieldCorrection)
+        self.addMenuItem(self.menuBatch, "Correct images ...", self.batch.cbBatchFlatFieldCorrection)
 
         self.menuBatch.addSeparator()
 
         # Measure.
-        self.addMenuItem(self.menuBatch, "Measure ...", 'Ctrl+M',
-                         self.batch.cbMeasureBatch)
+        self.addMenuItem(self.menuBatch, "Measure ...", self.batch.cbMeasureBatch)
 
         self.menubar.addMenu(self.menuBatch)
 
         # PLUGINS MENU ---------------------------------------------------------------------------
         # Install plugin.
-        self.addMenuItem(self.menuPlugins, "Install plugin ...", None,
-                         self.plugins.cbInstallPlugin)
+        self.addMenuItem(self.menuPlugins, "Install plugin ...", self.plugins.cbInstallPlugin)
 
         # Uninstall plugin.
-        self.addMenuItem(self.menuPlugins, "Uninstall plugin ...", None,
-                         self.plugins.cbUninstallPlugin)
+        self.addMenuItem(self.menuPlugins, "Uninstall plugin ...", self.plugins.cbUninstallPlugin)
 
         self.menuPlugins.addSeparator()
 
@@ -627,8 +619,7 @@ class PyJAMAS(QtCore.QObject):
 
         # PyJAMAS MENU ---------------------------------------------------------------------------
         # About.
-        self.addMenuItem(self.menuHelp, "About", None,
-                         self.about.cbAbout)
+        self.addMenuItem(self.menuHelp, "About", self.about.cbAbout)
 
         self.menubar.addMenu(self.menuHelp)
 
@@ -636,35 +627,46 @@ class PyJAMAS(QtCore.QObject):
         self.retranslateUi()
 
         # Init UI.
-        # self.io.cbLoadTimeSeries(PyJAMAS.logo_path)
         QtCore.QMetaObject.connectSlotsByName(self.MainWindow)
 
         self.gView.show()
         self.MainWindow.show()
 
     def initData(self) -> bool:
+        # Load the config file
+        try:
+            with open(self.config_path, 'r') as f:
+                self.config = tomlkit.load(f)
+        except FileNotFoundError:
+            with open(self.default_config_path, 'r') as f:
+                self.config = tomlkit.load(f)
+
         self._imageItem = None  # pixmap containing the image currently being displayed.
         self.zoom_index = 0  # index into PyJAMAS.zoom_factors to establish the zoom level.
-        self.brush_size: int = 3  # brush size to paint polylines
+        self.brush_size: int = self.config["options"].get("brush_size", PyJAMAS.brush_size)  # brush size to paint polylines
         self.show_annotations: bool = True  # display or not annotations.
-        self.display_fiducial_ids: bool = False  # display an identifier next to each fiducial
-        self.margin_size: int = 0  # margin size for cropping
-        self.crop_tracked_polyline: bool = False  # crop function takes polyline on one slice or all slices
-        self.fps: int = 7  # frames per second used to play the current sequence or when exporting as a movie
-        self.close_all_polylines: bool = False  # close all polylines loaded from file or load as open polylines those whose first and last point are not the same
+        self.display_fiducial_ids: bool = self.config["options"].get("display_fiducial_ids", PyJAMAS.display_fiducial_ids)  # display an identifier next to each fiducial
+        self.margin_size: int = self.config["options"].get("margin_size", PyJAMAS.margin_size)  # margin size for cropping
+        self.crop_tracked_polyline: bool = self.config["options"].get("crop_tracked_polyline", PyJAMAS.crop_tracked_polyline)  # crop function takes polyline on one slice or all slices
+        self.fps: int = self.config["options"].get("fps", PyJAMAS.fps)  # frames per second used to play the current sequence or when exporting as a movie
+        self.close_all_polylines: bool = self.config["options"].get("close_all_polylines", PyJAMAS.close_all_polylines)  # close all polylines loaded from file or load as open polylines those whose first and last point are not the same
         self.annotation_mode = PyJAMAS.no_annotations
         self.batch_classifier: BatchML = None
         self.gaussian_sigma: float = 0.0
         self.scale_factor: Tuple[float, float] = (1.0, 1.0)
+        self.balloon_crop_size: int = self.config["options"].get("balloon_crop_size", PyJAMAS.balloon_crop_size)  # crop size used to inflate balloons
+
+        self.livewire_margin: int = self.config["options"].get("livewire_margin", PyJAMAS.livewire_margin)
+        self.livewire_gaussian_sigma: float = self.config["options"].get("livewire_gaussian_sigma", PyJAMAS.livewire_gaussian_sigma)
+        self.livewire_shortest_path_fn: str = self.config["options"].get("livewire_shortest_path_fn", PyJAMAS.livewire_shortest_path_fn)
+        self.highlight_on_hover: bool = self.config["options"].get("highlight_on_hover", True)  # whether to highlight the object under the cursor
+        self.footprint: str = self.config["options"].get("footprint", "disk")
         self.active_element = None  # will hold the scene element that is under the mouse (for changing colour on hover)
-        self.highlight_on_hover: bool = True  # whether to highlight the object under the cursor
 
         self.min_pix_percentile: int = 0  # Lowest percentile of the pixel values to map to display value 0.
         self.max_pix_percentile: int = 100  # Highest percentile of the pixel values to map to display value 255.
 
-        self.cwd = os.getcwd()
         self.pjs_path, _ = os.path.split(os.path.realpath(__file__))
-
         self.plugin_list: List = []
         self.plugin_path_list: List = []
         self.plugin_path: str = os.path.join(self.pjs_path, 'plugins')
@@ -735,23 +737,33 @@ class PyJAMAS(QtCore.QObject):
 
         return True
 
-    def addMenuItem(self, themenu, theitemname, theshortcut, thecallbackfunction):
+    def addMenuItem(self, themenu, thetext, thecallbackfunction):
+        # Get a config name by replacing spaces with underscores and removing special characters
+        thename = re.sub('\s+', '_', thetext.strip())
+        thename = re.sub('[^A-Za-z0-9_]+', '', thename).lower()
+        if thename.endswith('_'):
+            thename = thename[:-1]
+
+        theshortcut = self.config["key_bindings"].get(thename, None)
+        if theshortcut is None:  # menu item that is not in the config, let's add it to the config file
+            self.config["key_bindings"][thename] = ""
+            theshortcut = ""
+
         newAction = QtGui.QAction(self.MainWindow)
         newAction.setEnabled(True)
-        newAction.setText(theitemname)
-        newAction.setIconText(theitemname)
-        newAction.setToolTip(theitemname)
-        newAction.setObjectName('action' + theitemname)
+        newAction.setText(thetext)
+        newAction.setIconText(thetext)
+        newAction.setToolTip(thetext)
+        newAction.setObjectName(thename)
         newAction.triggered.connect(thecallbackfunction)
 
-        if theshortcut is not None:
-            newAction.setShortcut(theshortcut)
+        newAction.setShortcut(theshortcut)
 
-            # There are other possible values for the ShortcutContext in Qt.ShortcutContext. This is key for shortcuts to work when setNativeMenuBar(True) is used.
-            newAction.setShortcutContext(QtCore.Qt.ShortcutContext.WidgetWithChildrenShortcut)
-            newShortcut = QtGui.QShortcut(QtGui.QKeySequence(theshortcut),
-                                              self.MainWindow)
-            newShortcut.activated.connect(thecallbackfunction)
+        # There are other possible values for the ShortcutContext in Qt.ShortcutContext. This is key for shortcuts to work when setNativeMenuBar(True) is used.
+        newAction.setShortcutContext(QtCore.Qt.ShortcutContext.WidgetWithChildrenShortcut)
+        newShortcut = QtGui.QShortcut(QtGui.QKeySequence(theshortcut), self.MainWindow)
+        newShortcut.activated.connect(thecallbackfunction)
+        newAction.setData({'shortcut': newShortcut, 'callback': thecallbackfunction})
 
         themenu.addAction(newAction)
 
@@ -1172,6 +1184,8 @@ class PyJAMAS(QtCore.QObject):
                 self.io.cbLoadClassifier(file_name_str)
             elif extension == PyJAMAS.plugin_extension:
                 self.plugins.cbInstallPlugin(file_name_str)
+            elif extension == PyJAMAS.config_extension:
+                self.io.cbLoadConfig(file_name_str)
 
         if pjfiles:
             self.io.cbLoadAnnotations(pjfiles, replace=replace_flag)
@@ -1243,6 +1257,31 @@ class PyJAMAS(QtCore.QObject):
 
         return True
 
+    def get_key_bindings(self):
+        key_bindings = {}
+        for this_menu in self.menubar.children():
+            if type(this_menu) == QtWidgets.QToolButton:
+                continue  # the menubar also has a tool button that is not one of the dropdown menus
+            these_shortcuts = {}
+            for this_action in self.get_menu_children(this_menu):
+                these_shortcuts[this_action.text()] = this_action.shortcut().toString()
+            key_bindings[this_menu.title()] = these_shortcuts
+        return key_bindings
+
+    def get_menu_children(self, menu: QtWidgets.QMenu) -> list:
+        menu_actions = menu.actions()
+        menu_children = []
+        for this_action in menu_actions:
+            if this_action.isSeparator():  # Separator
+                continue
+            elif menu.menuInAction(this_action) is not None:  # Submenu
+                menu_children.extend(self.get_menu_children(menu.menuInAction(this_action)))
+            else:  # Action connected to a callback function
+                menu_children.append(this_action)
+        return menu_children
+
+
+
     def paint_active_element(self, x: int, y: int) -> QtWidgets.QAbstractGraphicsShapeItem:
         if self.annotation_mode in self.fiducial_highlight_modes:
             object_types = QtWidgets.QGraphicsEllipseItem
