@@ -25,6 +25,7 @@ from pyjamas.rimage.rimcore import rimage
 from pyjamas.rimage.rimutils import rimutils
 import pyjamas.rutils as rutils
 from pyjamas.rannotations.rpolyline import RPolyline
+import skimage.morphology as skm
 from skimage.morphology import binary_erosion, binary_dilation, disk, label
 
 
@@ -106,89 +107,106 @@ class PJSEventFilter(QtCore.QObject):
                     self.pjs.removeFiducial(self.x, self.y, self.pjs.curslice, pushundo=True)
 
             elif self.pjs.annotation_mode == PyJAMAS.rectangle_annotations:
-                if event.type() == QtCore.QEvent.Type.MouseButtonPress:
-                    # Store first coordinate if left click ...
-                    if event.buttons() == QtCore.Qt.MouseButton.LeftButton:
-                        self.pjs._poly_ = [self.x, self.y]
+                if type(self.pjs._poly_) == list:
+                    if event.type() == QtCore.QEvent.Type.MouseButtonPress:
+                        # Store first coordinate if left click ...
+                        if event.buttons() == QtCore.Qt.MouseButton.LeftButton:
+                            self.pjs._poly_ = [self.x, self.y]
 
-                    # ... or delete polygon if right click.
-                    elif event.buttons() == QtCore.Qt.MouseButton.RightButton:
-                        self.pjs.removePolyline(self.x, self.y,
-                                                self.pjs.curslice, pushundo=True)  # CAN WE JUST HAVE ONE removeAnnotationItem function?
-
-                elif event.type() == QtCore.QEvent.Type.MouseMove and event.buttons() == QtCore.Qt.MouseButton.LeftButton:  # Mouse move events will occur only when a mouse button is pressed down, unless mouse tracking has been enabled with QWidget.setMouseTracking()
-                    if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
-                        self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
-
-                    # If shift is not pressed, draw a rectangle, a square if it is pressed.
-                    modifierPressed = QtWidgets.QApplication.keyboardModifiers()
-                    if (modifierPressed & QtCore.Qt.KeyboardModifier.ShiftModifier) != QtCore.Qt.KeyboardModifier.ShiftModifier:
-                        x = self.x
-                        y = self.y
-                    else:
-                        deltaX = self.x - self.pjs._poly_[0]
-                        deltaY = self.y - self.pjs._poly_[1]
-
-                        if abs(deltaX) >= abs(deltaY):
-                            x = self.x
-                            y = self.pjs._poly_[1] + numpy.sign(deltaY) * abs(deltaX)
-                        else:
-                            y = self.y
-                            x = self.pjs._poly_[0] + numpy.sign(deltaX) * abs(deltaY)
-
-                    self.pjs._agraphicsitem_ = self.pjs.drawRectangle(self.pjs._poly_[0], self.pjs._poly_[1], x, y)
-                    self.pjs.statusbar.showMessage(
-                        '{0}/{1}\t({2}, {3}) -> w: {4}, h: {5}'.format(str(self.pjs.curslice + 1),
-                        str(self.pjs.n_frames), str(
-                        self.pjs._poly_[0]), str(
-                        self.pjs._poly_[1]), str(abs(self.pjs._poly_[0] - x) + 1),
-                        str(abs(self.pjs._poly_[1] - y) + 1)))
-
-                elif event.type() == QtCore.QEvent.Type.MouseButtonRelease and self.pjs._poly_ != []:
-                    if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
-                        self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
-
-                    x0 = self.pjs._poly_[0]
-                    y0 = self.pjs._poly_[1]
-
-                    modifierPressed = QtWidgets.QApplication.keyboardModifiers()
-                    if (modifierPressed & QtCore.Qt.KeyboardModifier.ShiftModifier) != QtCore.Qt.KeyboardModifier.ShiftModifier:
-                        x1 = self.x
-                        y1 = self.y
-                    else:
-                        deltaX = self.x - self.pjs._poly_[0]
-                        deltaY = self.y - self.pjs._poly_[1]
-
-                        if abs(deltaX) >= abs(deltaY):
-                            x1 = self.x
-                            y1 = self.pjs._poly_[1] + numpy.sign(deltaY) * abs(deltaX)
-                        else:
-                            y1 = self.y
-                            x1 = self.pjs._poly_[0] + numpy.sign(deltaX) * abs(deltaY)
-
-                    if x0 > x1:
-                        x1, x0 = x0, x1
-                    if y0 > y1:
-                        y1, y0 = y0, y1
-
-                    self.pjs._poly_ = []
-                    self.pjs.addPolyline([[x0, y0], [x0, y1], [x1, y1], [x1, y0], [x0, y0]], self.pjs.curslice,
-                                         pushundo=True)
-
-            elif self.pjs.annotation_mode == PyJAMAS.polyline_annotations:
-                if event.type() == QtCore.QEvent.Type.MouseButtonPress:
-                    # Store coordinate if left click ...
-                    if event.buttons() == QtCore.Qt.MouseButton.LeftButton:
-                        self.pjs._poly_.append([self.x, self.y])
+                        # ... or delete polygon if right click.
+                        elif event.buttons() == QtCore.Qt.MouseButton.RightButton:
+                            self.pjs.removePolyline(self.x, self.y,
+                                                    self.pjs.curslice, pushundo=True)  # CAN WE JUST HAVE ONE removeAnnotationItem function?
 
+                    elif event.type() == QtCore.QEvent.Type.MouseMove and event.buttons() == QtCore.Qt.MouseButton.LeftButton:  # Mouse move events will occur only when a mouse button is pressed down, unless mouse tracking has been enabled with QWidget.setMouseTracking()
                         if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
                             self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
 
-                        self.pjs._agraphicsitem_, _ = self.pjs.drawPath(self.pjs._poly_)
+                        # If shift is not pressed, draw a rectangle, a square if it is pressed.
+                        modifierPressed = QtWidgets.QApplication.keyboardModifiers()
+                        if (modifierPressed & QtCore.Qt.KeyboardModifier.ShiftModifier) != QtCore.Qt.KeyboardModifier.ShiftModifier:
+                            x = self.x
+                            y = self.y
+                        else:
+                            deltaX = self.x - self.pjs._poly_[0]
+                            deltaY = self.y - self.pjs._poly_[1]
 
-                    # Delete last point if middle click
-                    elif event.buttons() == QtCore.Qt.MouseButton.RightButton and self.pjs._poly_ != [] and len(self.pjs._poly_) >= 2:
-                        self.pjs._poly_ = self.pjs._poly_[0:-1]
+                            if abs(deltaX) >= abs(deltaY):
+                                x = self.x
+                                y = self.pjs._poly_[1] + numpy.sign(deltaY) * abs(deltaX)
+                            else:
+                                y = self.y
+                                x = self.pjs._poly_[0] + numpy.sign(deltaX) * abs(deltaY)
+
+                        self.pjs._agraphicsitem_ = self.pjs.drawRectangle(self.pjs._poly_[0], self.pjs._poly_[1], x, y)
+                        self.pjs.statusbar.showMessage(
+                            '{0}/{1}\t({2}, {3}) -> w: {4}, h: {5}'.format(str(self.pjs.curslice + 1),
+                            str(self.pjs.n_frames), str(
+                            self.pjs._poly_[0]), str(
+                            self.pjs._poly_[1]), str(abs(self.pjs._poly_[0] - x) + 1),
+                            str(abs(self.pjs._poly_[1] - y) + 1)))
+
+                    elif event.type() == QtCore.QEvent.Type.MouseButtonRelease and self.pjs._poly_ != []:
+                        if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
+                            self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
+
+                        x0 = self.pjs._poly_[0]
+                        y0 = self.pjs._poly_[1]
+
+                        modifierPressed = QtWidgets.QApplication.keyboardModifiers()
+                        if (modifierPressed & QtCore.Qt.KeyboardModifier.ShiftModifier) != QtCore.Qt.KeyboardModifier.ShiftModifier:
+                            x1 = self.x
+                            y1 = self.y
+                        else:
+                            deltaX = self.x - self.pjs._poly_[0]
+                            deltaY = self.y - self.pjs._poly_[1]
+
+                            if abs(deltaX) >= abs(deltaY):
+                                x1 = self.x
+                                y1 = self.pjs._poly_[1] + numpy.sign(deltaY) * abs(deltaX)
+                            else:
+                                y1 = self.y
+                                x1 = self.pjs._poly_[0] + numpy.sign(deltaX) * abs(deltaY)
+
+                        if x0 > x1:
+                            x1, x0 = x0, x1
+                        if y0 > y1:
+                            y1, y0 = y0, y1
+
+                        self.pjs._poly_ = []
+                        self.pjs.addPolyline([[x0, y0], [x0, y1], [x1, y1], [x1, y0], [x0, y0]], self.pjs.curslice,
+                                             pushundo=True)
+                else:
+                    self.pjs._poly_ = []
+                    return False
+            elif self.pjs.annotation_mode == PyJAMAS.polyline_annotations:
+                if type(self.pjs._poly_) == list:
+                    if event.type() == QtCore.QEvent.Type.MouseButtonPress:
+                        # Store coordinate if left click ...
+                        if event.buttons() == QtCore.Qt.MouseButton.LeftButton:
+                            self.pjs._poly_.append([self.x, self.y])
+
+                            if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
+                                self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
+
+                            self.pjs._agraphicsitem_, _ = self.pjs.drawPath(self.pjs._poly_)
+
+                        # Delete last point if right click
+                        elif event.buttons() == QtCore.Qt.MouseButton.RightButton and self.pjs._poly_ != [] and len(self.pjs._poly_) >= 1:
+                            self.pjs._poly_ = self.pjs._poly_[0:-1]
+                            if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
+                                self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
+
+                                apoly = self.pjs._poly_.copy()
+                                apoly.append([self.x, self.y])
+
+                                self.pjs._agraphicsitem_, _ = self.pjs.drawPath(apoly)
+                        # ... or delete polygon if right click.
+                        elif event.buttons() == QtCore.Qt.MouseButton.RightButton:
+                            self.pjs.removePolyline(self.x, self.y, self.pjs.curslice, pushundo=True)
+
+                    # Redraw polyline as the user moves the cursor.
+                    elif event.type() == QtCore.QEvent.Type.MouseMove and self.pjs._poly_ != []:
                         if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
                             self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
 
@@ -196,39 +214,30 @@ class PJSEventFilter(QtCore.QObject):
                             apoly.append([self.x, self.y])
 
                             self.pjs._agraphicsitem_, _ = self.pjs.drawPath(apoly)
-                    # ... or delete polygon if right click.
-                    elif event.buttons() == QtCore.Qt.MouseButton.RightButton:
-                        self.pjs.removePolyline(self.x, self.y, self.pjs.curslice, pushundo=True)
 
-                # Redraw polyline as the user moves the cursor.
-                elif event.type() == QtCore.QEvent.Type.MouseMove and self.pjs._poly_ != []:
-                    if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
-                        self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
+                    # Add polyline as mouse button is released.
+                    elif event.type() == QtCore.QEvent.Type.MouseButtonDblClick and self.pjs._poly_ != []:
+                        if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
+                            self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
+
+                        # Polylines need to have at least two points.
+                        if len(self.pjs._poly_) <= 1:
+                            self.pjs._poly_ = []
+                            return False
 
                         apoly = self.pjs._poly_.copy()
-                        apoly.append([self.x, self.y])
 
-                        self.pjs._agraphicsitem_, _ = self.pjs.drawPath(apoly)
+                        # If shift is not pressed, go back to origin.
+                        modifierPressed = QtWidgets.QApplication.keyboardModifiers()
+                        if (modifierPressed & QtCore.Qt.KeyboardModifier.ShiftModifier) != QtCore.Qt.KeyboardModifier.ShiftModifier:
+                            apoly.append([self.pjs._poly_[0][0], self.pjs._poly_[0][1]])
 
-                # Add polyline as mouse button is released.
-                elif event.type() == QtCore.QEvent.Type.MouseButtonDblClick and self.pjs._poly_ != []:
-                    if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
-                        self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
-
-                    # Polylines need to have at least two points.
-                    if len(self.pjs._poly_) <= 1:
                         self.pjs._poly_ = []
-                        return False
-
-                    apoly = self.pjs._poly_.copy()
-
-                    # If shift is not pressed, go back to origin.
-                    modifierPressed = QtWidgets.QApplication.keyboardModifiers()
-                    if (modifierPressed & QtCore.Qt.KeyboardModifier.ShiftModifier) != QtCore.Qt.KeyboardModifier.ShiftModifier:
-                        apoly.append([self.pjs._poly_[0][0], self.pjs._poly_[0][1]])
+                        self.pjs.addPolyline(apoly, self.pjs.curslice, pushundo=True)
 
+                else:
                     self.pjs._poly_ = []
-                    self.pjs.addPolyline(apoly, self.pjs.curslice, pushundo=True)
+                    return False
 
 
             elif self.pjs.annotation_mode == PyJAMAS.delete_fiducials_outside_polyline:
@@ -292,7 +301,7 @@ class PJSEventFilter(QtCore.QObject):
                            self.pjs._agraphicsitem_, _ = self.pjs.drawPath(self.pjs._poly_)
 
                         # Delete last point if right click and in the middle of drawing.
-                        elif event.buttons() == QtCore.Qt.MouseButton.RightButton and self.pjs._poly_ != [] and len(self.pjs._poly_) >= 2:
+                        elif event.buttons() == QtCore.Qt.MouseButton.RightButton and self.pjs._poly_ != [] and len(self.pjs._poly_) >= 1:
                             self.pjs._poly_ = self.pjs._poly_[0:-1]
                             if self.pjs._agraphicsitem_ and self.pjs._agraphicsitem_.scene():
                                 self.pjs.gScene.removeItem(self.pjs._agraphicsitem_)
@@ -587,11 +596,12 @@ class PJSEventFilter(QtCore.QObject):
         except IndexError:
             return False
 
+        footprint = getattr(skm, self.pjs.footprint, skm.disk)
         if event.buttons() == QtCore.Qt.MouseButton.LeftButton:  # Dilation
-            obj_mask = binary_dilation(obj_mask, disk(self.pjs.brush_size))
+            obj_mask = binary_dilation(obj_mask, footprint(self.pjs.brush_size))
         elif event.buttons() == QtCore.Qt.MouseButton.RightButton:  # Erosion
             if self.pjs.brush_size > 1:
-                obj_mask = binary_erosion(obj_mask, disk(self.pjs.brush_size - 1))
+                obj_mask = binary_erosion(obj_mask, footprint(self.pjs.brush_size - 1))
             else:
                 return False
 
