@@ -389,7 +389,7 @@ class RCBBatchProcess(RCallback):
                 for anindex, aslice in enumerate(img):
                     tmp_img: numpy.ndarray = aslice.copy()
 
-                    if crop_flag and nrows < tmp_img.shape[0] and ncols < tmp_img.shape[1]:
+                    if crop_flag and nrows <= tmp_img.shape[0] and ncols <= tmp_img.shape[1]:
                         tmp_img = tmp_img[crop_start_rows:crop_start_rows + nrows,
                                   crop_start_cols:crop_start_cols + ncols]
 
@@ -1334,7 +1334,8 @@ class RCBBatchProcess(RCallback):
             code = f"ax = sns.lineplot(x='time (min)', y='{a_measurement}', hue='experimental group', data=all_data, errorbar='{parameters.get('errorbar', RCBBatchProcess.ERRORBAR_BM)}', err_style='{parameters.get('err_style_value', RCBBatchProcess.ERR_STYLE_VALUE_BM)}', lw={RCBBatchProcess.LINE_WIDTH_BM}, palette=a_palette)\n" \
                    f"handles, _ = ax.get_legend_handles_labels()\n" \
                    f"ax.legend(handles=handles, labels={parameters.get('names')}, frameon=False)\n" \
-                   f"sns.despine()"
+                   f"sns.despine()\n" \
+                   f"plt.show()"
             nb['cells'].append(nbf.v4.new_code_cell(code))
 
         return nb
@@ -1353,6 +1354,7 @@ class RCBBatchProcess(RCallback):
                f"    handles, _ = ax.get_legend_handles_labels()\n" \
                f"    ax.legend(handles=handles, labels={parameters.get('names')}, frameon=False)\n" \
                f"    sns.despine()\n" \
+               f"    plt.show()\n"\
                f"    return"
         nb['cells'].append(nbf.v4.new_code_cell(code))
 
@@ -1373,7 +1375,8 @@ class RCBBatchProcess(RCallback):
             code = f"ax = sns.lineplot(x='time (min)', y='{a_measurement}', hue='experimental group', data=all_data, estimator=None, units='experiment index', lw={RCBBatchProcess.LINE_WIDTH_BM}, legend='full', palette=a_palette)\n" \
                    f"handles, labels = ax.get_legend_handles_labels()\n" \
                    f"ax.legend(handles=handles, labels=labels, frameon=False)\n" \
-                   f"sns.despine()"
+                   f"sns.despine()\n" \
+                   f"plt.show()"
             nb['cells'].append(nbf.v4.new_code_cell(code))
 
             for a_group in self.group_labels:
@@ -1385,7 +1388,8 @@ class RCBBatchProcess(RCallback):
                        f"n_exp: int = len(numpy.unique(thedata['experiment index']).astype(int))\n" \
                        f"ax = sns.lineplot(x='time (min)', y='{a_measurement}', hue='experiment index', data=thedata, estimator=None, units='experiment index', lw={RCBBatchProcess.LINE_WIDTH_BM}, palette=sns.color_palette('husl', n_exp))\n" \
                        f"ax.legend([str(x) for x in range(n_exp)], frameon=False, title='{a_group}')\n" \
-                       f"sns.despine()"
+                       f"sns.despine()\n" \
+                       f"plt.show()"
                 nb['cells'].append(nbf.v4.new_code_cell(code))
 
         return nb
@@ -1404,6 +1408,7 @@ class RCBBatchProcess(RCallback):
                f"    handles, labels = ax.get_legend_handles_labels()\n" \
                f"    ax.legend(handles=handles[0:], labels=labels[0:], frameon=False)\n" \
                f"    sns.despine()\n" \
+               f"    plt.show()\n" \
                f"    \n" \
                f"    for a_group in {self.group_labels}:\n" \
                f"        plt.figure()\n" \
@@ -1412,6 +1417,7 @@ class RCBBatchProcess(RCallback):
                f"        ax = sns.lineplot(x='time (min)', y=metric, hue='experiment index', data=thedata, estimator=None, units='experiment index', lw={RCBBatchProcess.LINE_WIDTH_BM}, palette=sns.color_palette('husl', n_exp))\n" \
                f"        ax.legend([str(x) for x in range(n_exp)], frameon=False, title=a_group)\n" \
                f"        sns.despine()\n" \
+               f"        plt.show()\n" \
                f"    \n" \
                f"    return"
         nb['cells'].append(nbf.v4.new_code_cell(code))
@@ -1447,7 +1453,8 @@ class RCBBatchProcess(RCallback):
             code += f"handles, _ = ax.get_legend_handles_labels()\n" \
                     f"ax.legend(handles=handles, frameon=False)\n" \
                     f"ax = sns.stripplot(x='experimental group', y='{a_measurement}', data=thedata, color='k', alpha=0.75, size=6, dodge=False, jitter=0.05)\n" \
-                    f"sns.despine()"
+                    f"sns.despine()\n" \
+                    f"plt.show()"
             nb['cells'].append(nbf.v4.new_code_cell(code))
 
         return nb
@@ -1479,6 +1486,7 @@ class RCBBatchProcess(RCallback):
                 f"    ax.legend(handles=handles, frameon=False)\n" \
                 f"    ax = sns.stripplot(x='experimental group', y=metric, data=thedata, color='k', alpha=0.75, size=6, dodge=False, jitter=0.05)\n" \
                 f"    sns.despine()\n" \
+                f"    plt.show()\n" \
                 f"    return"
         nb['cells'].append(nbf.v4.new_code_cell(code))
 
