@@ -19,6 +19,8 @@
 import gzip
 import os
 import pickle
+import tomlkit
+import re
 from typing import List, Optional, Tuple
 
 import cv2
@@ -30,6 +32,7 @@ from sklearn.linear_model import LogisticRegression
 from sklearn.svm import SVC
 
 from pyjamas.external import pascal_voc_io
+import pyjamas.rimage.rimcore as rimcore
 from pyjamas.pjscore import PyJAMAS
 from pyjamas.pjscore import undo_modes
 from pyjamas.rcallbacks.rcallback import RCallback
@@ -1159,5 +1162,120 @@ class RCBIO(RCallback):
 
         return True
 
+    def cbLoadConfig(self, filename: Optional[str] = None, message: Optional[str] = "Load config file ..."):
+        """
+                Load a configuration file using tomlkit.
 
+                :param filename: file to open.
+                :param message: string to display as the dialog box title.
+                :return: True if the config file was loaded with no problems, False otherwise.
+                """
 
+        # Get file name.
+        if filename is None or filename is False:
+            file_dialog = QtWidgets.QFileDialog(None, message, self.pjs.config_path, filter="Config files (*.toml)")
+            file_dialog.setFileMode(file_dialog.FileMode.ExistingFile)
+            file_dialog.exec()
+            filename = file_dialog.selectedFiles()[0]
+            if filename == '':
+                return False
+
+        self.pjs.cwd = os.path.dirname(filename)
+        try:
+            with open(filename, 'r') as f:
+                self.pjs.config = tomlkit.load(f)
+            self.update_options()
+            self.update_key_bindings()
+            return True
+        except FileNotFoundError:
+            self.pjs.statusbar.showMessage("Could not find specified file.")
+            return False
+
+    def update_options(self):
+        """ Updates values for options using the values from self.pjs.config"""
+        for this_key in self.pjs.config['options'].keys():
+            if getattr(self.pjs, this_key, None) is not None:
+                setattr(self.pjs, this_key, self.pjs.config['options'][this_key])
+        return True
+
+    def update_key_bindings(self):
+        """Updates values for key bindings using the values from self.pjs.config"""
+        curr_bindings = self.pjs.get_key_bindings()
+        new_bindings = curr_bindings.copy()
+
+        # Get a translation dictionary to go from option names to config names (lowercase, replace spaces with _)
+        translation_dict = {}
+        for menu_name, this_menu in curr_bindings.items():
+            menu_keys = this_menu.keys()
+            translated_keys = []
+            for this_key in menu_keys:
+                this_key = re.sub('\s+', '_', this_key.strip())
+                this_key = re.sub('[^A-Za-z0-9_]+', '', this_key).lower()
+                if this_key.endswith('_'):
+                    this_key = this_key[:-1]
+                translated_keys.append(this_key)
+            translation_dict[menu_name] = translated_keys
+
+        for this_action, this_shortcut in self.pjs.config['key_bindings'].items():
+            for menu_name, this_menu in curr_bindings.items():
+                if this_action in translation_dict[menu_name]:
+                    this_key = list(this_menu.keys())[translation_dict[menu_name].index(this_action)]
+                    new_bindings[menu_name][this_key] = this_shortcut
+                    break
+        return self.pjs.options.configure_key_bindings(new_bindings)
+
+    def cbSaveConfig(self, filename: Optional[str] = None, message: Optional[str] = "Save config file ..."):
+        """
+                Save the current pjs.config as a .toml file.
+
+                :param filename: filepath to save to.
+                :param message: string to display as the dialog box title.
+                :return: True if the config file was saved with no problems, False otherwise.
+                """
+
+        # Get file name.
+        if filename is None or filename is False or filename == '':
+            file_dialog = QtWidgets.QFileDialog(None, message, self.pjs.config_path, filter="Config files (*.toml)")
+            file_dialog.setFileMode(file_dialog.FileMode.AnyFile)
+            file_dialog.setAcceptMode(file_dialog.AcceptMode.AcceptSave)
+            file_dialog.exec()
+            filename = file_dialog.selectedFiles()[0]
+
+        if filename == '':
+            filename = self.pjs.config_path
+
+        self.pjs.config_path = filename
+
+        if os.path.splitext(filename)[1] != ".toml":
+            filename = RUtils.set_extension(filename, ".toml")
+
+        self.update_config()
+        return self.save_config(filename)
+    
+    def update_config(self) -> bool:
+        # Update options fields
+        for this_key in self.pjs.config["options"].keys():
+            this_val = getattr(self.pjs, this_key, None)
+            if this_val is not None:
+                self.pjs.config["options"][this_key] = this_val
+
+        # Update keyboard shortcuts
+        for this_menu in self.pjs.menubar.children():
+            if type(this_menu) == QtWidgets.QToolButton:
+                continue
+            these_actions = self.pjs.get_menu_children(this_menu)
+            for this_action in these_actions:
+                self.pjs.config["key_bindings"][
+                    re.sub('[^A-Za-z0-9_]+', '', this_action.objectName())] = this_action.shortcut().toString()
+
+        return True
+    
+    def save_config(self, fpath: str) -> bool:
+        try:
+            with open(fpath, 'w') as f:
+                tomlkit.dump(self.pjs.config, f)
+        except:
+            self.pjs.statusbar.showMessage("Failed writing the config file.")
+            return False
+        return True
+    
\ No newline at end of file
